* RATS code to estimate 2-variable structural VAR as in Blanchard-Perotti
* written by James Hamilton March 2016
cal 1947 1 4
      * identifies as quarterly observations beginning 1947:Q1
compute capt = 2015:4
      * a variable capt is introduced to facilitate updating with later data
open data var2_data.csv
data(org=obs) 1947:1 capt datex gdpreal gdpnom taxnom
* col 1:  date
* col 2:  level of real GDP (2009 dollars)
* col 3:  level of nominal GDP
* col 4:  level of nominal personal income taxes collected

* convert GDP to annual growth rates
set gdpch = 400*log(gdpreal(t)/gdpreal(t-1))
* convert tax to growth of tax rate
set tax = 100*log(taxnom(t)/gdpnom(t))
set taxch = tax(t) - tax(t-1)

smpl 1947:1 capt

* ================================================================
* set NBER bus cycle dates if desired
set recessdum = 0
set recessdum 1948:4 1949:4 = 1
set recessdum 1953:2 1954:2 = 1
set recessdum 1957:3 1958:2 = 1
set recessdum 1960:2 1961:1 = 1
set recessdum 1969:4 1970:4 = 1
set recessdum 1973:4 1975:1 = 1
set recessdum 1980:1 1980:3 = 1
set recessdum 1981:3 1982:4 = 1
set recessdum 1990:3 1991:1 = 1
set recessdum 2001:1 2001:4 = 1
set recessdum 2007:4 2009:2 = 1
open styles darker.txt

open styles darker.txt
grparm(import=styles)

* plot data
spgraph(vfields=2)
smpl 1947:2 capt
graph(dates,header='Tax rate',shading=recessdum) 1
# tax
graph(dates,header='GDP growth',shading=recessdum) 1
# gdpch
spgraph(done)

* =================================================================
* estimate VAR and nonorthogonalized IRF

system(model=var2)
variables tax gdpch
lags 1 to 4
det constant
end(system)
estimate(sigma)

compute neqn = 2
compute nsteps = 20
declare vect[labels] implabel(neqn)
compute implabel=|| 'taxrate', 'GDP' ||
declare rect[series] impblk(neqn,1)

smpl 1 nsteps
impulse(result=impblk,model=var2) neqn nsteps * %identity(neqn)
do j=1,neqn
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Nonorthogonalized response of ' +implabel(i) +' to ' +implabel(j)
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,j)
end do
spgraph(done)
end do

set e1 = impblk(2,1)(t)
set e2 = impblk(2,2)(t)

* =======================================================================
* estimate structural VAR by numerical maximum likelihood

compute a12 = -0.72
nonlin a21
dec frml[rect] afrml
frml afrml = || 1.0, a12 | $
                a21, 1.0 ||
compute a21 = 0
cvmodel(iters=100,method=bfgs,trace,factor=afactor) %sigma afrml

compute aest = || 1.0, a12 | $
                a21, 1.0 ||
compute ainv = inv(aest)

display 'estimated value for A is '
display aest
display 'confirm that this diagonalizes sigma'
compute d = aest*%sigma*tr(aest)
display d
display 'estimated value of inv(A) is'
display ainv

smpl 1 nsteps
do j=1,neqn
impulse(model=var2, shock=%xcol(ainv,j), result=impblk) neqn nsteps
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j)
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,1)
if j == 1
     {
     set e3 = impblk(2,1)(t)
     }
end do i
spgraph(done)
end do j

spgraph(vfields=3)
graph(header='Nonorthgonalized response of GDP to taxrate',nodates,number=0,$
min=-0.4,max=1.0) 1
# e1
graph(header='Nonorthgonalized response of GDP to GDP',nodates,number=0,$
min=-0.4,max=1.0) 1
# e2
graph(header='Structural response of GDP to taxrate',nodates,number=0,$
min=-0.4,max=1.0) 1
# e3
spgraph(done)


* ======================================================================================
* Calculate MLE analytically using IV
compute a12 = -0.72
compute biv = %sigma(1,2) + a12*%sigma(2,2)
comute biv = biv/(%sigma(1,1) + a12*%sigma(1,2))
display 'Value of tax impact calculated by IV'
display biv

* ======================================================================================
* Calculate tax impact as function of assumed tax elasticity
compute capn = 300
smpl 1 capn
set elast = 0.0
set imp = 0.0
compute a12 = 0.0
do i = 1, capn
  compute biv = %sigma(1,2) + a12*%sigma(2,2)
  comute biv = biv/(%sigma(1,1) + a12*%sigma(1,2))
  set elast i i = a12
  set imp i i = biv
  compute a12 = a12 - 0.01

end do i
print(nodates) 1 capn elast imp
set elast2 = -elast(t)
scatter(style=lines) 1
# elast2 imp

* =======================================================
* calculate bounds

display 'variance matrix is'
display %sigma
display 'smallest effect if elasticity = 0'
compute small = %sigma(1,2)/%sigma(1,1)
display small
display 'biggest effect if elasticity is infinite'
compute asymp = %sigma(2,2)/%sigma(1,2)
display asymp
